import numpy as np
import inspect
import pyteos_air.teos_air as teos_air

def attribute_to_function(method_name,function_name,*args):
    return np.squeeze(getattr(teos_air,method_name)(function_name,*np.broadcast_arrays(*np.atleast_3d(*args))))



realm=__name__.split('.')[-2]
input_type=__name__.split('.')[-1]

from ... import pyteos_interface
import sys

desc={}
desc['entropy']="wet air entropy (J/K).\n\n\
At the freezing temperature, this function assumes that half of the condensate is in liquid phase and\
that the other half is in ice phase."
desc['cond_entropy']="wet air entropy with all the moisture in condensed phase (J/K)"
desc['temperatureequi']="equivalent temperature (K)"
desc['rh_wmo']="relative humidity using WMO definition"

__all__=desc.keys()

docstring="""
:param A: dry air massfraction (kg/kg)
:type A: np.array.
:param T: absolute temperature (K)
:type T: np.array.
:param p: total pressure (Pa)
:type p: np.array.
:returns: {0}

This is for wet air with ice and liquid.
"""

for function_name in __all__:
    def __tool(A,T,p):
        return pyteos_interface.attribute_to_function(realm+'_'+input_type,function_name,A,T,p)
    __tool.__name__=function_name
    __tool.__doc__=docstring.format(desc[function_name])
    setattr(sys.modules[__name__], function_name, __tool)



realm=__name__.split('.')[-2]
input_type=__name__.split('.')[-1]

from ... import pyteos_interface
import sys

desc={}
desc['ifl']="isentropic freezing level (Pa).\n\n\
For pressures higher than the **ifl**, there can be only liquid water."
desc['iml']="isentropic melting level (Pa).\n\n\
For pressures lower than the **iml**, there can be only ice."

__all__=desc.keys()

docstring="""
:param A: dry air massfraction (kg/kg)
:type A: np.array.
:param eta: wet air entropy obtained from :func:`pyteos_air.liq_ice_air.g.entropy` (J/K)
:type eta: np.array.
:returns: {0}

This is for wet air with ice and liquid.
"""

for function_name in __all__:
    def __tool(A,eta):
        return pyteos_interface.attribute_to_function(realm+'_'+input_type,function_name,A,eta)
    __tool.__name__=function_name
    __tool.__doc__=docstring.format(desc[function_name])
    setattr(sys.modules[__name__], function_name, __tool)



realm=__name__.split('.')[-2]
input_type=__name__.split('.')[-1]

from ... import pyteos_interface
import sys

desc={}
desc['temperature']="absolute temperature (K).\n\n\
Note that this function is not an exact inverse of :func:`pyteos_air.liq_ice_air.g.entropy`. \
At the freezing point, this function assumes that a combination of liquid water and ice will \
combine to make sure that the temperature is constant between the Isentropic Freezing level \
:func:`pyteos_air.liq_ice_air.il.ifl` and the Isentropic Melting Level :func:`pyteos_air.liq_ice_air.il.iml`"

__all__=desc.keys()

docstring="""
:param A: dry air massfraction (kg/kg)
:type A: np.array.
:param eta: wet air entropy obtained from :func:`pyteos_air.liq_ice_air.g.entropy` (J/K)
:type eta: np.array.
:param p: total pressure (Pa)
:type p: np.array.
:returns: {0}

This is for wet air with ice and liquid.
"""

for function_name in __all__:
    def __tool(A,eta,p):
        return pyteos_interface.attribute_to_function(realm+'_'+input_type,function_name,A,eta,p)
    __tool.__name__=function_name
    __tool.__doc__=docstring.format(desc[function_name])
    setattr(sys.modules[__name__], function_name, __tool)



realm=__name__.split('.')[-2]
input_type=__name__.split('.')[-1]

from ... import pyteos_interface
import sys

desc={}
desc['pottemp']="potential tempeature (K) at **pref**."
desc['pottempequisat']="saturation equivalent potential temperature (K) at **pref**."
desc['pottempequipseudo']="pseudo equivalent potential temperature (K) at **pref**.\n\n\
Integrates the pseudo adiabat until the temperature falls\
outside the range of validity of TEOS-10."
desc['pottempequi']="equivalent potential tempeature (K) at **pref**."

__all__=desc.keys()

docstring="""
:param A: dry air massfraction (kg/kg)
:type A: np.array.
:param T: absolute temperature (K)
:type T: np.array.
:param p: total pressure (Pa)
:type p: np.array.
:param pref: reference total pressure (Pa)
:type pref: np.array.
:returns: {0}

This is for wet air with ice and liquid.
"""

for function_name in __all__:
    def __tool(A,T,p,pref):
        return pyteos_interface.attribute_to_function(realm+'_'+input_type,function_name,A,T,p,pref)
    __tool.__name__=function_name
    __tool.__doc__=docstring.format(desc[function_name])
    setattr(sys.modules[__name__], function_name, __tool)
